package RSSReader::Gtk2UI::DiscoverFeed;

use strict;
use RSSReader::Category;
use RSSReader::Gtk2UI::Dialog;
use RSSReader::Fetcher::FindFeed;
use base ('RSSReader::Logger');

sub run {
    my $c = shift;
    my %a = @_;

    my $s = bless {}, $c; 

    $s->{_category}       = $a{category} || RSSReader::Category->new(id=>1);
    $s->{_step}           = 0;
    $s->{_display_widget} = undef;
    $s->{_display_area}   = new Gtk2::VBox;
    $s->{vbox} 		  = new Gtk2::VBox;
    $s->{vbox}->pack_start($s->{_display_area},1,1,1);
    $s->{vbox}->pack_start(Gtk2::HSeparator->new,0,0,0);
    $s->{vbox}->show_all();

    $s->{_button_box} = new Gtk2::HButtonBox();
    $s->{_button_box}->set_layout_default('end');
    $s->{_button_box}->set_spacing_default( 3);
    $s->{_button_box}->show;

    $s->{_prev_button} = Gtk2::Button->new_from_stock('gtk-go-back');
    $s->{_prev_button}->signal_connect('clicked', sub { $s->prev_step } );
    $s->{_button_box}->add($s->{_prev_button});

    $s->{_next_button} = Gtk2::Button->new_from_stock('gtk-go-forward');
    $s->{_next_button}->signal_connect('clicked', sub { $s->next_step } );
    $s->{_button_box}->add($s->{_next_button});

    $s->{_finish_button} = Gtk2::Button->new_from_stock('Finish');
    $s->{_finish_button}->signal_connect('clicked', sub { $s->finish } );
    $s->{_button_box}->add($s->{_finish_button});

    $s->{_cancel_button} = Gtk2::Button->new_from_stock('gtk-cancel');
    $s->{_cancel_button}->signal_connect('clicked', sub { $s->{_window}->destroy } );
    $s->{_button_box}->add($s->{_cancel_button});

    $s->{vbox}->pack_start($s->{_button_box},0,0,0);

    $s->{_window} = new Gtk2::Window("toplevel");
    $s->{_window}->set_default_size( 250, 200 );
    $s->{_window}->set_title("Syndigator Discover Feed Wizard");

    $s->{_window}->add($s->{vbox});

    $s->{_window}->show_all;
    $s->log_debug("Calling first step");
    $s->next_step(%a);
}

sub next_step {
    my $s = shift;
    my %a = @_;
    $s->log_debug("removing current widget");
    $s->log_debug("current step is ".$s->{_step});
    if ($s->{_step} > 0) {
    	$s->log_debug("Validating leap");
	my $func_val = "step_".$s->{_step}."_validate";
	my $val;
	eval { $val = $s->$func_val };
	return unless $val;
    }
    if ( defined($s->{_display_widget}) ) {
	$s->{_display_area}->remove($s->{_display_widget});
    }
    $s->log_debug("validated .. drawing next step");
    $s->{_step}++; 
    my $func = "step_".$s->{_step};
    eval { $s->$func(%a) };
    if ($@) { die $@ }
}

sub prev_step {
    my $s = shift;
    $s->log_debug("removing current widget");
    $s->log_debug("current step is ".$s->{_step});
    if ( defined($s->{_display_widget}) ) {
	$s->{_display_area}->remove($s->{_display_widget});
    }
    $s->log_debug("validated .. drawing next step");
    $s->{_step}--; 
    my $func = "step_".$s->{_step};
    eval { $s->$func };
    if ($@) { die $@ }
}


sub step_1 {
    my $s = shift;
    my %a = @_;

    $s->{_finish_button}->hide();
    $s->{_prev_button}->show();
    $s->{_prev_button}->set_sensitive(0);
    $s->{_next_button}->show();
    $s->{_next_button}->set_sensitive(1);

    unless ($s->{_step1_init}) {

	$s->{_step1_display} = new Gtk2::VBox(1,5);
    	$s->{_step1_align}   = new Gtk2::Alignment(0.5,0.5,5,5);
    	$s->{_step1_label}   = Gtk2::Label->new('Please insert a url and click "next". We will try to find a feed from that url'); 
    	$s->{_step1_align}->add($s->{_step1_label});
    	$s->{_step1_label}->set_line_wrap( 1 );
	$s->{_step1_display}->pack_start($s->{_step1_align},0,0,5);

	$s->{_step1_table} = new Gtk2::Table(2,2);
	$s->{_step1_url_label} = Gtk2::Label->new("Url: ");
	$s->{_step1_url_entry} = new Gtk2::Entry();
	$s->{_step1_url_entry}->set_text($a{url}) if $a{url};
	$s->{_step1_table}->attach($s->{_step1_url_label}, 0,1,0,1,[],[],10,0);
	$s->{_step1_table}->attach($s->{_step1_url_entry}, 1,2,0,1,['fill','expand'],[],10,0);
	$s->{_step1_display}->pack_start($s->{_step1_table},0,0,0);
	$s->{_step1_init} = 1;
    }
    
    $s->{_display_widget}= $s->{_step1_display};
    $s->{_display_area}->pack_start($s->{_display_widget},1,1,1);
    $s->{_display_area}->show_all;
    $s->{_step} = 1;
}

sub step_1_validate {
    my $s = shift;
    unless ($s->{_step1_url_entry}->get_text ne "" ) {
  	RSSReader::Gtk2UI::Dialog->run("Please fill the url entry");	
	return undef;
    }
    $s->{_given_url} = $s->{_step1_url_entry}->get_text;
    return 1;
}

sub step_2 {
    my $s = shift;
    
    #CALL THE FINDER NOW;
    my $result = RSSReader::Fetcher::FindFeed->run(url=>$s->{_given_url});
    
    $s->{_prev_button}->show();
    $s->{_prev_button}->set_sensitive(1);
    $s->{_next_button}->hide();
    $s->{_finish_button}->show();
    $s->{_finish_button}->set_sensitive(1);
    $s->{_step} = 2;

    if (!$result) {
    	$s->{_finish_button}->set_sensitive(0);
	my $label = new Gtk2::Label("Could not find a feed from url ".$s->{_given_url}.". Click \"back\" to try another url, or \"cancel\" to abort this operation");
    	$label->set_line_wrap( 1 );
	$s->{_display_widget}= $label;
	$s->{_display_area}->pack_start($s->{_display_widget},1,1,1);
	$s->{_display_area}->show_all;
	$s->{_find_result} = undef;
    }
    else {
	my $label = new Gtk2::Label("Found a feed: ".$result->{xml}->{channel}->{title}." (".$result->{url}.")\nPress 'Finish' to add this feed to your subscriptions.");
	$s->{_display_widget}= $label;
	$s->{_display_area}->pack_start($s->{_display_widget},1,1,1);
	$s->{_display_area}->show_all;
	$s->{_find_result} = $result;
    }
}

sub finish {
	my $s = shift;
	my $f = RSSReader::Feed->create( 	category       => $s->{_category},
				 		url            => $s->{_find_result}->{url},
				 		label	       => $s->{_find_result}->{xml}->{channel}->{title},
						fetch_interval => 0
				        );
	if (! $f ) {
		RSSReader::Gtk2UI::Dialog->run($@);
		$s->{_window}->destroy;
		return undef;
	}
	$s->{_window}->destroy;
	RSSReader::Gtk2UI->new()->get_feed($f);
}

1;
