package RSSReader::Gtk2UI::ImporterDruid;
use strict;
use RSSReader::Importer;
use RSSReader::Gtk2UI::Dialog;
use Gtk2::SimpleList;
use Data::Dumper;
use base qw( RSSReader::Logger);

sub run {
    my $c        = shift;
    my $importer = shift;

    my $w = new Gtk2::Window;
    $w->set_title("Syndigator ".$importer->name()." importer");
    $w->set_default_size("300","300");
    my $druid = Gnome2::Druid->new(); 
    my $s = bless { _importer => $importer, _druid=>$druid}, $c;
    $s->{_druid} = $druid;

    my $start = Gnome2::DruidPageEdge->new('start');
    $start->set_title("Syndigator ".$importer->name()." importer: ");
    #$start->set_logo_bg_color({ 'red'    => 0, 'green'  => 0, 'blue'   => 0x7000 });
    $start->set_text($importer->description);
    $start->show();
    $druid->append_page($start);
    $druid->show();
    $druid->set_page($start);

    push @{$s->{_pages}}, $start;

    #info collecting
    my $page1 = Gnome2::DruidPageStandard->new();
    $page1->set_title("Information gathering");
    $page1->show();
    my $label_into = new Gtk2::Label();
    $label_into->set_markup("<b>The following information is required to import the information</b>\n\n");
    #$page1->vbox->pack_start($label_into,0,1,20); 
    $page1->append_item("", $label_into, "");
    $label_into->set_alignment(0.5,0.0);
    $label_into->show();

    foreach my $config_opt ( @{$importer->config_options} ) {
	$c->log_debug("Found an option ".Dumper($config_opt) );	

	if ($config_opt->{type} eq 'text') { 
		my $entry = Gtk2::Entry->new();
		if ($config_opt->{default}) { $entry->append_text($config_opt->{default}) }
		$s->{_options_to_run}->{$config_opt->{name}} =  { type   => 'text',
								  widget => $entry, };
		$page1->append_item($config_opt->{desc}, $entry, "");

	} elsif ( $config_opt->{type} eq 'file' ) {
		my $fentry = Gnome2::FileEntry->new(1,'Select '.$config_opt->{desc});
		if ($config_opt->{default}) { $fentry->gtk_entry->append_text($config_opt->{default}) }
		$s->{_options_to_run}->{$config_opt->{name}} =  { type   => 'file',
								  widget => $fentry };
		$page1->append_item($config_opt->{desc}, $fentry, "");
	}

	$page1->signal_connect('next', sub { $_[2]->info_gathered }, $s );
    }
    $s->{_page_1} = $page1;
    $druid->append_page($page1);

    #PAGE 2: information presentation.
    my $page2 = Gnome2::DruidPageStandard->new();
    $page2->set_title("Collected information");
    $page2->show();
    $druid->append_page($page2);
    $page2->signal_connect( 'next', sub { $_[2]->{_druid}->set_page($_[2]->{_page_final}); return 1 }, $s );
    $s->{_page_2}->{widget} = $page2;

    #PAGE FINAL
    my $pagefinal = Gnome2::DruidPageEdge->new('finish');
    $pagefinal->set_text("");
    $s->{_page_final} = $pagefinal;
    $s->{_page_final}->show_all;
    $s->{_page_final}->signal_connect('finish',  sub { $_[2]->feeds_add; $w->destroy }, $s );
    $s->{_page_final}->signal_connect('prepare', sub { $_[2]->pre_add_msg}, $s );
    $druid->append_page($pagefinal);

    #PAGE NO FEEDS FOUND
    my $page_no_feeds = Gnome2::DruidPageEdge->new('finish');
    $page_no_feeds->set_text("No feeds where found by this importer.");
    $s->{_page_no_feeds} = $page_no_feeds;
    $s->{_page_no_feeds}->show_all;
    $s->{_page_no_feeds}->signal_connect('back', sub { shift; my $s = shift; $s->{_druid}->set_page($s->{_page_1}); return 1 }, $s);
    $s->{_page_no_feeds}->signal_connect('finish', sub { $w->destroy }, $s );
    $druid->append_page($page_no_feeds);
    

    $druid->signal_connect('cancel', sub { $w->destroy } );
    $s->{_current_page} = 0;

    $w->add($druid);
    $w->show();
}
sub feeds_presentation {
    my $s = shift;
    $s->log_debug("Should present feeds");
	
    if ($s->{_page_2}->{label}) {
	    my $l = new Gtk2::Label("The following feeds were found by ".$s->{_importer}->name." importer.");
	    $s->{_page_2}->{widget}->append_item("",$l,"");
	    $s->{_page_2}->{label};
    }
	
    # Create a ScrolledWindow for the list
    if (! $s->{_page_2}->{list} ) {
	    my $list_scrolled_win = new Gtk2::ScrolledWindow( undef, undef );
	    $list_scrolled_win->set_policy( 'automatic', 'automatic' );
	    $list_scrolled_win->set_size_request (-1, 150);
	    $list_scrolled_win->show();
  	    $s->{_page_2}->{list} = Gtk2::SimpleList->new('Feed Name'=>'text');
 	    $list_scrolled_win->add( $s->{_page_2}->{list} );
	    $list_scrolled_win->show_all;
	    $s->{_page_2}->{list}->get_selection->set_mode ('multiple');
    	    $s->{_page_2}->{widget}->append_item("Please select the feeds that you wish to add to your Syndigator feed list.",$list_scrolled_win,"");
	    print STDERR $s->{_page_2}->{widget}->get_child()->get_child();
    }
    @{$s->{_page_2}->{list}->{data}} = ();
    my $k = 0;
    foreach my $f ( @{$s->{_feeds_collected}->{feeds}} ) {
    		print Dumper $f;
		push @{$s->{_page_2}->{list}->{data}}, [ $f->{label} ]
    }
    $s->{_druid}->set_page($s->{_page_2}->{widget});
    return 1;
}

sub info_gathered {
    my $s = shift;

    $s->log_debug("Should call importer->run to check for options inconsistence");

    my %o;

    foreach my $option (keys %{$s->{_options_to_run}}) {
	if ($s->{_options_to_run}->{$option}->{type} eq 'text') {
		$o{$option} = $s->{_options_to_run}->{$option}->{widget}->get_text;
	} elsif ( $s->{_options_to_run}->{$option}->{type} eq 'file') {
		$o{$option} = $s->{_options_to_run}->{$option}->{widget}->gtk_entry->get_text;
	}
    }

    $s->log_debug("Will pass this options to fetch : ".Dumper(\%o));
    my $res = $s->{_importer}->fetch(%o);
    if (!$res) {
	$s->log_debug("Fetch failed: $@");
	RSSReader::Gtk2UI::Dialog->run($@, 'error');
	$s->{_feeds_collected} = {};
    }
    else {
	$s->log_debug("Fetch result: ".Dumper($res));
	$s->{_feeds_collected} = $res;
	if (scalar(@{$s->{_feeds_collected}->{feeds}})) {
		$s->feeds_presentation();
	}
	else {
		$s->{_druid}->set_page($s->{_page_no_feeds});
	}
    }
    
    return 1;
}

sub pre_add_msg {
    my $s = shift;
    if ( defined($s->{_page_2}->{list}->get_selected_indices) ) {
	    my @selection = $s->{_page_2}->{list}->get_selected_indices; 
	    $s->{_page_final}->set_text(scalar(@selection)." feeds will be added to your Syndigator.\nPress finish to continue");

    } else {
	    $s->{_page_final}->set_text("No feeds where selected.\nPress back to select the feeds to import\nor cancel to abort the import operation");
    }
}

sub feeds_add {
    my $s = shift;
    $s->log_debug("Should add feeds");
    if ($s->{_feeds_collected}
        && scalar(@{$s->{_feeds_collected}->{feeds}})
	&& defined($s->{_page_2}->{list}->get_selected_indices())
	) {
	
	my %starter = ();

	my @selection = $s->{_page_2}->{list}->get_selected_indices; 
        $s->log_debug("Feeds selected : ".Dumper \@selection);
	
	my @to_add = ();
	foreach my $i (@selection) {
		push @to_add, $s->{_feeds_collected}->{feeds}->[$i];
	}

	$s->{_importer}->add(feeds=>\@to_add, categories=>$s->{_feeds_collected}->{categories});
    }

    $s->{_druid}->set_page($s->{_page_final});

    return 1;
}


1;
