package RSSReader::Gtk2UI::Tree;
use Data::Dumper;
use strict;
use RSSReader::Category;
use RSSReader::Feed;
use RSSReader::Gtk2UI;
use RSSReader::Gtk2UI::Feed;
use RSSReader::Gtk2UI::RenameCategoryWindow;
use RSSReader::Gtk2UI::AddACategory;
use RSSReader::Gtk2UI::AddAFeedToCategory;
use RSSReader::Gtk2UI::DeleteAFeed;;
use RSSReader::Gtk2UI::DeleteACategory;
use RSSReader::Gtk2UI::DiscoverFeedSilent;
use RSSReader::Gtk2UI::IconFactory;
use RSSReader::Gtk2UI::Preferences;
use base qw( RSSReader::Notifier RSSReader::Logger );

our $_tree;

sub new {
        my $c = shift;
        return $_tree if defined($_tree);

        $_tree = bless {}, $c;
        return $_tree;
}

sub widget {
        my $s = shift;
        $s->_init() unless $s->{_init_done};
        return $s->{_widget};
}

sub tree_widget {
        my $s = shift;
        $s->_init() unless $s->{_init_done};
        return $s->{_tree_widget};
}

sub show {
        my $s = shift;
        $s->_init() unless $s->{_init_done};
        $s->{_widget}->show();

        return;
}


sub select_next_unread {
        my $s 	  = shift;

	my $selection = $s->{_tree_widget}->get_selection;
	my $iter = $selection->get_selected;
	
	my $p = $s->{_tree_model}->get_string_from_iter($iter);
	
	return $s->_go($iter, $p, 1);
}

sub _who_should_give_to {
	my $s = shift;
	my $current = shift;

	my $parent = $s->{_tree_model}->iter_parent($current);
	if ($parent && (my $next = $s->{_tree_model}->iter_next($parent)) ) { 
	  return $next;
	} elsif ($parent) {
	  return $s->_who_should_give_to($parent);
	} else {
	  return $current;
	}
}

sub _go {
	my $s = shift;
	my $current = shift;
	my $first = shift;
	my $first_pass = shift;
	my $model = $s->{_tree_model};
	my $old = $current;

	my $path_current = $model->get_string_from_iter($current);

	$s->log_spam("$first $path_current");

	return undef if ($path_current eq $first && (!$first_pass));
	
        my ($type, $id, $unread) = ($s->{_tree_model}->get($current,0), $s->{_tree_model}->get($current,1), $s->{_tree_model}->get($current,3));
	my $object;
	if ($type eq 'CATEGORY') {
	       $object = RSSReader::Category->new(id=>$id);
	} else {
	       $object = RSSReader::Feed->new(id=>$id);
	}

	if (scalar(@{$object->unread_messages}) && $type eq 'FEED') {
	       $s->{_tree_widget}->get_selection->select_iter($current);
	       return $object;
	}
	elsif (scalar(@{$object->unread_messages}) && (! $s->{_tree_widget}->row_expanded($model->get_path($current)) ) )  {
               $s->{_tree_widget}->get_selection->select_iter($current);
               return $object;
	}
	elsif ($model->iter_has_child($current) && $s->{_tree_widget}->row_expanded($model->get_path($current)) ) {
		return $s->_go($model->iter_nth_child($current,0), $first);

	}
	elsif ( my $next = $model->iter_next($current) ){
		return $s->_go($next ,$first);
	}
	else {
		my $next = $s->_who_should_give_to($current);
		return $s->_go($next, $first);
	}

}

sub _init {
        my $s = shift;

	$s->{_prefs} = RSSReader::Gtk2UI::Preferences->new();

        # Create a ScrolledWindow for the tree
        my $tree_scrolled_win = new Gtk2::ScrolledWindow( undef, undef );
        $tree_scrolled_win->set_policy( 'automatic', 'automatic' );
        $tree_scrolled_win->show();

        # Create root tree

	$s->{_tree_model} = Gtk2::TreeStore->new (qw/ 
					 Glib::String  
					 Glib::String  
                                         Glib::String
                                         Glib::String /);
					 #type
					 #id
					 #label
					 #number of unread

	$s->{_tree_model}->set_sort_column_id(2,'ascending');
					 
	$s->{_tree_widget} = Gtk2::TreeView->new_with_model($s->{_tree_model});
	$s->{_tree_widget}->set_rules_hint(1);
	my $treeview = $s->{_tree_widget};
	$s->{_tree_widget}->set_headers_visible(0);

	my $renderer_pix = Gtk2::CellRendererPixbuf->new;
	my $renderer = Gtk2::CellRendererText->new;
	$renderer->set('weight-set' => undef, 'xalign'=>0, 'xpad'=>0, 'ypad'=>0 );
        $renderer->signal_connect('edited', sub {$_[3]->_label_edited(@_) }, $s);
	#label
	my $col_label = Gtk2::TreeViewColumn->new();
	my $col_offset = $s->{_tree_widget}->insert_column( $col_label,0);
	$col_label->pack_start($renderer_pix,0);
	$col_label->pack_start($renderer,1);
	$col_label->set_cell_data_func(
				$renderer_pix,
				sub {
                                       	my ($treecol, $cell, $model, $iter, $col_num) = @_;
                                       	my $type  = $model->get ($iter, 0);
					my $id    = $model->get ($iter, 1);
					if ( $type eq 'FEED' )  {
						my $feed = RSSReader::Feed->new(id=>$id);
						my $pixbuf;
						if ($feed->fetches_failed) {
							$cell->set(xalign=>1.0, xpad=>2, ypad=>0, visible=>1, stock_id => 'gtk-dialog-warning');
						} elsif ($feed->notify_new_messages) {
							$cell->set(xalign=>1.0, xpad=>2, ypad=>0, visible=>1, stock_id => 'syndigator-notify');
						} else {
							$cell->set(xalign=>1.0, xpad=>2, ypad=>0, visible=>1, stock_id => 'syndigator-item');
						}
					}	
					else {
						$cell->set(xalign=>1.0, ypad=>0, visible=>0);
					}
		             },
			     undef
			     );
	$col_label->set_cell_data_func(
				$renderer,
				sub {
                                       my ($treecol, $cell, $model, $iter, $col_num) = @_;
                                        my $type  = $model->get ($iter, 0);
					my $id    = $model->get ($iter, 1);
					my $label = $model->get ($iter, 2); 
					my $unread = $model->get ($iter, 3);
					if ( $unread > 0 && $type eq 'FEED')  {
						$cell->set(xalign=>0.0, editable=>0, text => "$label ($unread)", weight => 600 );
					}	
					elsif ($unread > 0) {
						$cell->set(xalign=>0.0, editable=>0, text => "$label", weight => 600 );
					} else {
						$cell->set(xalign=>0.0, editable=>0, text => "$label", weight=>0);
					}
			             },
				undef
	);

	$renderer = Gtk2::CellRendererText->new;
	my $col_offset2 = $s->{_tree_widget}->insert_column_with_data_func(
				2,
				"unread_msg",
				$renderer,
				sub {
                                       my ($treecol, $cell, $model, $iter, $col_num) = @_;
                                        my $type = $model->get ($iter, 0);
					my $id = $model->get ($iter, 1);
					if ($type eq 'FEED') {
						my $unread = $model->get ($iter, 3);
						if ($unread)  {
							$cell->set( text=> "($unread)", weight => 600);
						}	
						else {
							$cell->set (text => "", weight=>0);
						}
					}
					else {
							$cell->set (text => "", weight=>0);
					}
			             },
				undef
	);
	my $cl = $s->{_tree_widget}->get_column(1);
	$cl->set_visible(0);

	my $root_cat = RSSReader::Category->new(id=>1);
	$s->category_draw( category=>$root_cat);

	my $selection = $treeview->get_selection; 
	$selection->signal_connect('changed', sub { $_[1]->tree_selection($_[0]) },$s );
	$selection->select_iter($s->{_tree_model}->get_iter_first);

	my $l = new Gtk2::Label("Please Choose a Feed or Category");
	
	$s->{_widget}      = $tree_scrolled_win;
	$s->{_widget}->add($s->{_tree_widget});
	$s->{_widget}->show;
	$s->{_tree_widget}->signal_connect('button-press-event', 
					   sub { my ($w, $e, $s) = @_; 
					   return $s->_button_press(widget=>$w, event=>$e) },$s);
	
	my @target_table = (
        	{'target' => "STRING",     'flags' => ['same-app'], 'info' => 0},
        	{'target' => "text/plain", 'flags' => ['same-app'], 'info' => 0},
        	{'target' => "text/plain", 'flags' => [],           'info' => 0},
		{'target' => "application/syndigator", 'flags' => ['same-app'], 'info' => 2},
        	{'target' => "application/x-rootwin-drop", 'flags' => ['same-app'], 'info' => 1},
	);

	$s->{_tree_widget}->enable_model_drag_source (['button1_mask', 'button3_mask'], ['copy', 'move'], $target_table[2] );
	$s->{_tree_widget}->signal_connect ("drag_data_get", sub { $s->source_drag_data_get(@_) } );

	$s->{_tree_widget}->enable_model_drag_dest (['copy', 'move'], @target_table[0..2]);
	$s->{_tree_widget}->signal_connect("drag_data_received", sub { $s->drag_data_received(@_) } );
	
	$s->{_tree_widget}->signal_connect("motion-notify-event", sub {$s->_tree_motion(@_) } );
	$s->{_tree_widget}->signal_connect("leave-notify-event", sub {$s->_tree_left(@_) } );

	$s->{_tree_widget}->signal_connect("row-expanded" , sub { $s->row_expanded(@_) } );
	$s->{_tree_widget}->signal_connect("row-collapsed", sub { $s->row_collapsed(@_) } );

	$s->{_init_done} = 1;
	return 1;
}

sub row_expanded {
	my ($s, $tv, $iter) = @_;
	my ($type, $id) = $s->{_tree_model}->get($iter,0,1);		
	return 1 unless $type eq 'CATEGORY';
	$s->{_prefs}->set('category_'.$id,"expand", 1);
	return 1;
}

sub row_collapsed {
	my ($s, $tv, $iter) = @_;
	my ($type, $id) = $s->{_tree_model}->get($iter,0,1);		
	return 1 unless $type eq 'CATEGORY';
	print STDERR "Collapsed category $id\n";
	$s->{_prefs}->set('category_'.$id,"expand", 0);
	my $c = RSSReader::Category->new(id=>$id);
	foreach my $cat ( @{$c->get_sub_categories} ) {
		$s->{_prefs}->set('category_'.$cat->id,"expand", 0);
        }


	return 1;
}

sub _tree_tip_timeout {
	my $s = shift;
	$s->log_debug("Should show tip");
	my ($w, $e) = @_;
	my ($path) = $s->{_tree_widget}->get_path_at_pos($s->{_helper_tip_rec}->x, $s->{_helper_tip_rec}->y);;
	return unless $path;
	my $iter = $s->{_tree_model}->get_iter($path); 

	my ($type, $id) = ($s->{_tree_model}->get($iter,0), $s->{_tree_model}->get($iter,1));
	$s->log_debug("I GOT PATH: $path ($type, $id)");
	return unless $type eq 'FEED';

	my $feed = RSSReader::Feed->new(id=>$id);

	$s->{_helper_tip_window} = Gtk2::Window->new('popup');
	#$s->{_helper_tip_window}->set_resizable(0);
	$s->{_helper_tip_window}->set_name("gtk-tooltips");
	$s->{_helper_tip_window}->ensure_style;
	$s->{_helper_tip_window}->set_app_paintable(1);

	my $scr_w = Gtk2::Gdk->screen_width();
	my $scr_h = Gtk2::Gdk->screen_height();
	my ($sc, $x, $y) = Gtk2::Gdk::Display->get_default->get_pointer;
	$y += 5;

	$s->{_helper_tip_window}->move($x, $y);
	
	my $label_text = '<span size="larger"><b>'.$feed->label.'</b></span>'."\n<b>Status:</b>";
	if ($feed->fetches_failed) {
		$label_text .= "Last fetch <i>failed</i>, ".$feed->fetches_failed_reason; 
	}
	else {
		$label_text .= "Ok";
	}
	if ($feed->notify_new_messages) {
		$label_text .= "\n<i>This feed is marked for notification</i>";
	}
	
	my $label = new Gtk2::Label;
	$label->set_markup($label_text);
	$label->set_justify('left');
	$label->show;
	my $hbox = new Gtk2::HBox;
	my $vbox = new Gtk2::VBox;
	my $image;
	if ($feed->fetches_failed) {
		$image = Gtk2::Image->new_from_stock('gtk-dialog-warning', 'dialog');
	} elsif ($feed->notify_new_messages) {
		$image = Gtk2::Image->new_from_stock('syndigator-notify', 'dialog');
	} else {
		$image = Gtk2::Image->new_from_stock('syndigator-item', 'dialog');
	}
	$hbox->pack_start($image,0,0,0);
	$hbox->pack_start($vbox,1,1,1);
	$vbox->pack_start($label,1,1,1);
	$hbox->show_all;
	$s->{_helper_tip_window}->set_border_width(5);
	$s->{_helper_tip_window}->add($hbox);

	my $style = $s->{_helper_tip_window}->style;	
	$s->{_helper_tip_window}->realize;
	$style->paint_flat_box($s->{_helper_tip_window}->window(), 'normal', 'out', Gtk2::Gdk::Rectangle->new(2,2,$s->{_helper_tip_window}->get_size), $s->{_helper_tip_window},"tt",0,0,-1,-1 );
	
	$s->{_helper_tip_window}->show_all;

	return undef;
}

sub _tree_motion {
	my $s = shift;
	my ($w, $e) = @_;
	if ($s->{_helper_tip_timeout}) {
		Glib::Source->remove($s->{_helper_tip_timeout});	
		delete $s->{_helper_tip_timeout};
		delete $s->{_helper_tip_rec};
		if ( $s->{_helper_tip_window} ) {
			$s->{_helper_tip_window}->destroy;
			delete $s->{_helper_tip_window};
		}
	}
	my $path = $s->{_tree_widget}->get_path_at_pos($e->x, $e->y);
	return unless $path;
	$s->{_helper_tip_rec}     = $s->{_tree_widget}->get_cell_area($path, $s->{_tree_widget}->get_column(0) );
	$s->{_helper_tip_timeout} = Glib::Timeout->add(500, sub { $s->_tree_tip_timeout(@_) } );
	return undef;
}

sub _tree_left {
	my $s = shift;
	my ($w, $e) = @_;
	$s->log_spam("motion leave");
	Glib::Source->remove($s->{_helper_tip_timeout}) if $s->{_helper_tip_timeout};
	$s->{_helper_tip_window}->destroy if $s->{_helper_tip_window};
	delete $s->{_helper_tip_window};
	delete $s->{_helper_tip_rec};
	return undef;	
}

sub source_drag_data_get {
	my $s = shift;
	my ($w, $context, $data) = @_;

	my $selection = $s->{_tree_widget}->get_selection;
        my $iter = $selection->get_selected();
        return unless $iter;
	$s->{_tree_model}->get_string_from_iter($iter);


	$data->set($data->target, 8, "SynDigatorMove::".$s->{_tree_model}->get_string_from_iter($iter) ); 
}

sub drag_data_received {
	my $s = shift;
	my ($widget, $context, $x, $y, $data, $info, $time) = @_;

	my ($path) = $s->{_tree_widget}->get_path_at_pos ($x,$y);
	my $iter_dest = $s->{_tree_model}->get_iter($path);
	my ($type, $id) = ($s->{_tree_model}->get($iter_dest,0), $s->{_tree_model}->get($iter_dest,1));
	if ($type eq 'FEED') {
		my $iter_dest =  $s->{_tree_model}->iter_parent($iter_dest);
		($type, $id) = ($s->{_tree_model}->get($iter_dest,0), $s->{_tree_model}->get($iter_dest,1));
	}
	my $dest = RSSReader::Category->new(id=>$id);

	if ($data->data =~ /^SynDigatorMove::/) {
		#this is a syndigator move
		my $d = $data->data;
		$d =~ s/^SynDigatorMove:://;
		my $iter_orig = $s->{_tree_model}->get_iter_from_string($d);
		my ($orig_type, $orig_id) = ($s->{_tree_model}->get($iter_orig,0), $s->{_tree_model}->get($iter_orig,1));

		print "SOurce is $orig_type, $orig_id\n";
		my $orig;
		if ($orig_type eq 'FEED') { $orig = RSSReader::Feed->new(id=>$orig_id)     }
		else                      { $orig = RSSReader::Category->new(id=>$orig_id) }
		
		my $res = $orig->move(to=>$dest);
		if ($res) {
			$context->finish (1, 0, $time);
		}
		else {
			$context->finish (0, 0, $time);	
		}

	} else {
		#anything else drop
		RSSReader::Gtk2UI::DiscoverFeedSilent->run(category=>$dest, url=>$data->data);
	}

	$context->finish (1, 0, $time);
	return;
}

sub _button_press {
    my $s = shift;
    my %a = @_;
                                                                                                                                            
    my $selection = $s->{_tree_widget}->get_selection;
    my $selected = $selection->get_selected_rows();
    $s->log_debug("button pressed:  ".$a{event}->button." as ".$a{event}->type);
                                                                                                                                            
    if ($a{event}->button == 3) {
        print "its a rigth click :)\n";
	my ($p) = $s->{_tree_widget}->get_path_at_pos($a{event}->x, $a{event}->y);
	$selection->select_path($p) unless $p == $selected;
	my $iter = $s->{_tree_model}->get_iter($p);
	my ($type, $id) = ($s->{_tree_model}->get($iter,0), $s->{_tree_model}->get($iter,1));
	$s->_popup($type, $id);

	return 1;
    }

    return undef;
}

sub tree_selection {
	my $s = shift;
	my $selection = shift;

	my $iter = $selection->get_selected();
	return unless $iter;
	$s->{_tree_widget}->scroll_to_cell($s->{_tree_model}->get_path($iter), undef, 0,1,1);
	my ($type, $id) = ($s->{_tree_model}->get($iter,0), $s->{_tree_model}->get($iter,1));
	if ($type eq 'FEED') {
		$s->notify('view_selection', view => RSSReader::Gtk2UI::Feed->new(id=>$id));
	} else {
		$s->notify('view_selection', view => RSSReader::Gtk2UI::Category->new(id=>$id));
	}

}

sub _add_feed {
	my $s = shift;
	my %a = @_;

	$a{feed} = $a{where} unless $a{feed};

	die "I need a feed" unless $a{feed};

	$s->log_spam("New feed was created");
	my $parent 	= $a{feed}->location;
	my $parent_iter = undef;
	if (defined($parent)) {
		$s->log_spam("And has as parent $parent");
		$parent_iter = $s->_find_iter(category=>$parent ); 
		die "Could not find an iter for my parent" unless $parent_iter;
	}

	my $iter = $s->{_tree_model}->append($parent_iter);
	$s->feed_draw(feed=>$a{feed}, iter=>$iter);

	#expand parent if not expanded
	$s->{_tree_widget}->expand_to_path( $s->{_tree_model}->get_path($iter) );
	$s->{_tree_widget}->get_selection->select_iter($iter);
}

sub _category_moved {
	my $s = shift;
	my %a = @_;
	
	my $cat = $a{where};
	my $from = $a{from};
	my $to   = $a{to};
	
	if ($to) {
		my $i = $s->_find_iter(category=>$to);	    
		$s->_update_iter(iter=>$i, category=>$to);
	}
	if ($from) {
		my $i = $s->_find_iter(category=>$from);	    
		$s->_update_iter(iter=>$i, category=>$from);
	}

	my $iter    = $s->_find_iter(category=>$cat); 

	$s->_category_deleted(where=>$cat);
	$s->_add_category(category=>$cat);
}

sub _feed_moved {
	my $s = shift;
	my %a = @_;
	
	my $feed = $a{where};
	my $from = $a{from};
	my $to   = $a{to};
	
	if ($to) {
		my $i = $s->_find_iter(category=>$to);	    
		$s->_update_iter(iter=>$i, category=>$to);
	}
	if ($from) {
		my $i = $s->_find_iter(category=>$from);	    
		$s->_update_iter(iter=>$i, category=>$from);
	}
	
	my $iter    = $s->_find_iter(feed=>$feed); 

	$s->_feed_deleted(where=>$feed);
	$s->_add_feed(feed=>$feed);
}

sub _add_category {
	my $s = shift;
	my %a = @_;

	$a{category} = $a{where} unless $a{category};

	die "I need a category" unless $a{category};

	my $parent 	= $a{category}->location;
	my $parent_iter = undef;
	if (defined($parent)) {
		$parent_iter = $s->_find_iter(category=>RSSReader::Category->new(id=>$parent) ); 
		die "Could not find an iter for my parent" unless $parent_iter;
	}

	my $iter = $s->{_tree_model}->append($parent_iter);
	$s->category_draw(category=>$a{category}, iter=>$iter);
	#expand parent if not expanded
	$s->{_tree_widget}->expand_to_path( $s->{_tree_model}->get_path($iter) );
	$s->{_tree_widget}->get_selection->select_iter($iter);
}

sub category_draw {
	my $s = shift;
	my %a = @_;

	my $iter = $a{iter} || $s->{_tree_model}->append(undef);

	$s->_update_iter(iter=>$iter, category=>$a{category});

	$a{category}->notify_me(event=>'label_change'              , caller=>$s, call=>'_redraw_category');
        $a{category}->notify_me(event=>'new_message'               , caller=>$s, call=>'_redraw_category');
        $a{category}->notify_me(event=>'message_marked_read'       , caller=>$s, call=>'_redraw_category');
        $a{category}->notify_me(event=>'message_marked_unread'     , caller=>$s, call=>'_redraw_category');
        $a{category}->notify_me(event=>'message_deleted'           , caller=>$s, call=>'_redraw_category');
        $a{category}->notify_me(event=>'feed_deleted_after'        , caller=>$s, call=>'_redraw_category');
        $a{category}->notify_me(event=>'category_moved_after'      , caller=>$s, call=>'_category_moved');
        $a{category}->notify_me(event=>'new_feed'                  , caller=>$s, call=>'_add_feed');
        $a{category}->notify_me(event=>'new_category'              , caller=>$s, call=>'_add_category');
        $a{category}->notify_me(event=>'category_deleted'          , caller=>$s, call=>'_category_deleted');

	my $expand = $s->{_prefs}->get("category_".$a{category}->id, "expand");
	print STDERR "Got expand $expand\n";
	$expand = 1 unless defined($expand);
	if ($expand) {
	 print STDERR "Expanding ".$a{category}->label." ".$a{category}->id."\n" if $expand;
	 $s->{_tree_widget}->expand_to_path($s->{_tree_model}->get_path($iter));
	 $s->{_tree_widget}->expand_row($s->{_tree_model}->get_path($iter),1);
	}

        foreach my $cat_feed (@{$a{category}->get_feeds()}) {
		$s->feed_draw(feed=>$cat_feed,iter=>$s->{_tree_model}->append($iter), expand=>$expand);
        };

	foreach my $cat ( @{$a{category}->get_sub_categories} ) {
		$s->category_draw(category=>$cat, iter=>$s->{_tree_model}->append($iter));
	}
}


sub _redraw_category {
	my $s = shift;
	my %a = @_;
	$a{where} = $a{category} unless $a{where};

	my $iter = $s->_find_iter(category=>$a{where});
	return unless $iter;
	$s->_update_iter(iter=>$iter, category=>$a{where});
}


sub feed_draw {
	my $s = shift;
	my %a = @_;
	$s->_update_iter(iter=>$a{iter}, feed=>$a{feed});

	if ($a{expand}) {
	  $s->{_tree_widget}->expand_to_path($s->{_tree_model}->get_path($a{iter}));
	}

        $a{feed}->notify_me(event=>'message_marked_read',              caller=>$s, call=>'_redraw_feed' );
        $a{feed}->notify_me(event=>'message_marked_unread',            caller=>$s, call=>'_redraw_feed' );
        $a{feed}->notify_me(event=>'message_deleted',                  caller=>$s, call=>'_redraw_feed' );
        $a{feed}->notify_me(event=>'new_message',                      caller=>$s, call=>'_redraw_feed' );
        $a{feed}->notify_me(event=>'feed_fetch_failed',                caller=>$s, call=>'_redraw_feed' );
        $a{feed}->notify_me(event=>'feed_fetch_success',               caller=>$s, call=>'_redraw_feed' );
        $a{feed}->notify_me(event=>'label_changed',                    caller=>$s, call=>'_redraw_feed' );
        $a{feed}->notify_me(event=>'feed_moved_after',                 caller=>$s, call=>'_feed_moved'  );
        $a{feed}->notify_me(event=>'feed_deleted',                     caller=>$s, call=>'_feed_deleted');
        $a{feed}->notify_me(event=>'notify_new_messages_changed',      caller=>$s, call=>'_redraw_feed' );
	return 1;
}

sub _feed_deleted {
	my $s = shift;
	my %a = @_;
	my $iter = $s->_find_iter(type=>'FEED', feed=>$a{where});
	return unless $iter;
	$s->{_tree_model}->remove( $iter );
	my $selection = $s->{_tree_widget}->get_selection;
	$selection->select_iter($s->{_tree_model}->get_iter_first);
}
sub _category_deleted {
	my $s = shift;
	my %a = @_;
	my $iter = $s->_find_iter(type=>'CATEGORY', category=>$a{where});
	return unless $iter;
	$s->{_tree_model}->remove( $iter );
	my $selection = $s->{_tree_widget}->get_selection;
	$selection->select_iter($s->{_tree_model}->get_iter_first);
}

sub _redraw_feed {
	my $s = shift;
	my %a = @_;

	$a{where} = $a{feed} unless $a{where};

	$s->log_debug("Redrawing feed ".$a{where}->label);

	my $iter = $s->_find_iter(feed=>$a{where});
	return unless $iter;
	$s->log_debug("Found an iter for feed");
	$s->_update_iter(iter=>$iter, feed=>$a{where});
}

sub _update_iter {
	my $s = shift;
	my %a = @_;

	if ($a{category}) {
		$s->log_debug("updating category ".$a{category}->label);
		$s->{_tree_model}->set($a{iter}, 0,'CATEGORY', 1, $a{category}->id, 2, $a{category}->label,3,scalar(@{$a{category}->unread_messages}) );
	}
	else {
		$s->log_debug("updating feed ".$a{feed}->label);
		$s->{_tree_model}->set($a{iter}, 0,'FEED', 1, $a{feed}->id, 2, $a{feed}->label, 3, scalar( @{$a{feed}->unread_messages} ) );
	}
	return 1;
}

sub _find_iter {
	my $s = shift;
	my %a = @_;

	$a{iter} = $s->{_tree_model}->get_iter_first() unless defined $a{iter};
	while ($a{iter}) {
	     my ($type, $id) = $s->{_tree_model}->get($a{iter},0,1);		
	     return $a{iter} if ( $a{feed}     && $type eq 'FEED'     && ($id eq $a{feed}->id)     );
	     return $a{iter} if ( $a{category} && $type eq 'CATEGORY' && ($id eq $a{category}->id) );
	     my $children = $s->{_tree_model}->iter_children($a{iter});
	     if ($children) {
		     my $search_child = $s->_find_iter(iter => $children, feed=>$a{feed}, category=>$a{category} );
		     return $search_child if $search_child;
	     }
	     
	     $a{iter} = $s->{_tree_model}->iter_next($a{iter});
	}
	return undef;
}

sub _label_edited {
	
	my $s = shift;
	my $renderer = shift;
	my $path = shift;
	my $new_label = shift;

	my $iter = $s->{_tree_model}->get_iter_from_string($path);
	my ($type, $id) = $s->{_tree_model}->get($iter,0,1);		
	if ($type eq 'CATEGORY') {
		$s->log_debug("Category label edited");
		my $cat = RSSReader::Category->new(id=>$id);
		$cat->set_label(new_label=>$new_label);
		$s->_redraw_category(category=>$cat);
	}
	else {
		$s->log_debug("Feed label edited");
		my $feed = RSSReader::Feed->new(id=>$id);
		$feed->label($new_label);
		$s->_redraw_feed(feed=>$feed);
	}
	
}

sub _popup {
     my $s = shift;
     my $type = shift;
     my $id = shift;

     my @items = ();
     my $object;
     if ($type eq 'FEED') {
	$object = RSSReader::Feed->new(id=>$id);
     }
     else {
	$object = RSSReader::Category->new(id=>$id);
     }

     if ($type eq 'FEED') {
	@items = (
	        [
                	'/Properties',
	                undef,
	                sub { RSSReader::Gtk2UI::Feed->new(id=>$id)->do_show_preferences },
	                1,
	                '<StockItem>',
  	              'gtk-preferences'
        	],
        	[
        	        '/Delete',
	                undef,
			sub { RSSReader::Gtk2UI::DeleteAFeed->run(RSSReader::Feed->new(id=>$id)); },
           	     	1,
               		 '<StockItem>',
                	'gtk-remove'
        	],
	        [
         	       '/separator',
             		undef,
             	   	undef,
                	1,
                	'<Separator>',
                	''
        	],
        	[
        	        '/Mark all messages as read',
	                undef,
        	       	sub { RSSReader::Feed->new(id=>$id)->mark_all_read },
           	     	1,
               		'<StockItem>',
                	''
        	],
        	[
        	        '/Mark all messages as unread',
	                undef,
        	       	sub { RSSReader::Feed->new(id=>$id)->mark_all_unread },
           	     	1,
               		'<StockItem>',
                	''
        	],
	        [
         	       '/separator2',
             		undef,
             	   	undef,
                	1,
                	'<Separator>',
                	''
        	],
        	[
        	        '/Launch feed site on browser',
	                undef,
        	       	sub { RSSReader::Gtk2UI->launch_url($object->source_link)},
           	     	1,
               		'<StockItem>',
                	'gtk-jump-to'
        	],
        	[
        	        '/Launch feed RSS on browser',
	                undef,
        	       	sub { RSSReader::Gtk2UI->launch_url($object->url) },
           	     	1,
               		'<StockItem>',
                	'gtk-jump-to'
        	],
        	[
        	        '/Validate feed RSS',
	                undef,
        	       	sub { RSSReader::Gtk2UI->launch_url($object->validate_rss_url)},
           	     	1,
               		'<StockItem>',
                	'gtk-jump-to'
        	],

	);
     } else {
     	@items = (
        	[
        	        '/Add a feed ...',
	                undef,
        	       	 sub { RSSReader::Gtk2UI::AddAFeedToCategory->run( category=>RSSReader::Category->new(id=>$id)); },
           	     	1,
               		 '<StockItem>',
                	'gtk-add'
        	],
        	[
        	        '/Add a category ...',
	                undef,
        	       	sub { RSSReader::Gtk2UI::AddACategory->run(category=>RSSReader::Category->new(id=>$id)); },
			1,
               		'<StockItem>',
                	'gtk-add'
        	],
        	[
        	        '/Rename ...',
	                undef,
        	       	sub { RSSReader::Gtk2UI::RenameCategoryWindow->run(category=>RSSReader::Category->new(id=>$id)) },
           	     	1,
               		 '<StockItem>',
                	'gtk-add'
        	],
        	[
        	        '/Delete',
	                undef,
			sub { RSSReader::Gtk2UI::DeleteACategory->run(RSSReader::Category->new(id=>$id)); },
           	     	1,
               		 '<StockItem>',
                	'gtk-remove'
        	],
	        [
         	       '/separator',
             		undef,
             	   	undef,
                	1,
                	'<Separator>',
                	''
        	],
        	[
        	        '/Mark all messages as read',
	                undef,
        	       	sub { RSSReader::Category->new(id=>$id)->mark_all_read },
           	     	1,
               		'<StockItem>',
                	''
        	],
        	[
        	        '/Mark all messages as unread',
	                undef,
        	       	sub { RSSReader::Category->new(id=>$id)->mark_all_unread },
           	     	1,
               		'<StockItem>',
                	''
        	],
	);

     }

    my $factory = Gtk2::ItemFactory->new('Gtk2::Menu', '<treepopup>', undef);
    $factory->create_items(1, @items);

    if ($type eq 'CATEGORY' && $id == 1) {
	$factory->get_item('<treepopup>/Delete')->hide();
    }
                                                                                                                                            
    my $menu = $factory->get_widget('<treepopup>');
    $menu->popup(undef, undef, undef,undef, 0, Gtk2->get_current_event_time());
}
1;

