package RSSReader::Importer::OPML;

use strict;
use RSSReader::Category;
use RSSReader::Feed;
use RSSReader::Importer;
use Data::Dumper;
use base qw( RSSReader::Logger RSSReader::Importer );


sub new {
  my $c = shift;
  my %a = @_;
  my $opml_file = $a{file} || $ENV{HOME}.'/default.opml';

  return bless { opml_file => $opml_file}, $c
}

sub fetch {
  my $s = shift;
  my %a = @_;

  my @feeds;

  $s->log_spam("Importing OPML feeds...");

  my $opml_file = $a{file};

  unless (open(FH, '<', $opml_file)) {
    $s->log_warn("Can't open OPML file: $!...");
    $@ = "Could not find OPML file";
    return undef;
  }
  my $import ='';
  local $_;
  while (<FH>) { 
    $import.=$_ ;
  }             
  close(FH);

  my @imports = split( /<outline/ , $import);
  foreach my $outline (@imports){
     my ($url) = $outline =~ /xmlurl="(.*?)"/i;
     my ($label) = $outline =~ /text="(.*?)"/;
     ($label) = $outline =~ /title="(.*?)"/ unless $label;
     next unless ($url && $label);
     push @feeds, { 
                     label          => $label,
                     url            => $url
                   };
     
  }
  $s->log_spam("...Fetched\n");
  my %to_return = (
                    feeds      => \@feeds,
                    categories => []
                  );

  return wantarray ? %to_return : \%to_return;

}


sub id { return "importer_generic_opml"; }

sub name { return "OPML"; }

sub config_options {
  my $s=shift; 
  return [{
            default => $s->{opml_file}, 
            name    => 'file', 
            desc    => 'generic OPML file', 
            type    => 'file'
         }];
}

sub description {
	return "Import feeds from an OPML format file.\nYou can export subscriptions on this format from Straw and AmphetaDesk.\n\nFor more information on this format go to http://www.opml.org/ .";
}

RSSReader::Importer->register( id => __PACKAGE__->id, package => __PACKAGE__ );
1;
